@@ -32,6 +32,7 @@ var gExceptionPaths = [
   "chrome://activity-stream/content/data/content/assets/remote/",
   "chrome://activity-stream/content/data/content/assets/mobile-download-qr-new-user-cn.svg",
   "chrome://activity-stream/content/data/content/assets/mobile-download-qr-existing-user-cn.svg",
+  "chrome://activity-stream/content/data/content/assets/mr-amo-collection.svg",
   "chrome://activity-stream/content/data/content/assets/person-typing.svg",
   "chrome://browser/content/assets/moz-vpn.svg",
   "chrome://browser/content/assets/vpn-logo.svg",
@@ -162,6 +163,9 @@ var allowlist = [
   // toolkit/mozapps/extensions/AddonContentPolicy.cpp
   { file: "resource://gre/localization/en-US/toolkit/global/cspErrors.ftl" },
 
+  // toolkit/components/antitracking/bouncetrackingprotection/BounceTrackingProtection.cpp
+  { file: "resource://gre/localization/en-US/toolkit/global/antiTracking.ftl" },
+
   // The l10n build system can't package string files only for some platforms.
   {
     file: "resource://gre/chrome/en-US/locale/en-US/global-platform/mac/accessible.properties",
@@ -287,9 +291,7 @@ var allowlist = [
 
   // Referenced programmatically
   { file: "chrome://browser/content/backup/BackupManifest.1.schema.json" },
-
-  // Bug 1892002
-  { file: "resource://app/modules/TopSites.sys.mjs" },
+  { file: "chrome://browser/content/backup/ArchiveJSONBlock.1.schema.json" },
 ];
 
 if (AppConstants.NIGHTLY_BUILD) {
@@ -301,10 +303,6 @@ if (AppConstants.NIGHTLY_BUILD) {
       // A debug tool that is only available in Nightly builds, and is accessed
       // directly by developers via the chrome URI (bug 1888491)
       { file: "chrome://browser/content/backup/debug.html" },
-
-      // The Transformers.js prod lib is not used in Nightly builds
-      { file: "chrome://global/content/ml/transformers.js" },
-      { file: "chrome://global/content/ml/ort.js" },
     ]
   );
 }
@@ -362,9 +360,6 @@ const ignorableAllowlist = new Set([
 
   // dom/media/gmp/GMPParent.cpp
   "resource://gre/gmp-clearkey/0.1/manifest.json",
-
-  // Bug 1351669 - obsolete test file
-  "resource://gre/res/test.properties",
 ]);
 for (let entry of ignorableAllowlist) {
   allowlist.add(entry);
