@@ -50,6 +50,7 @@ describe("ASRouter", () => {
   let FakeToolbarBadgeHub;
   let FakeMomentsPageHub;
   let ASRouterTargeting;
+  let gBrowser;
   let screenImpressions;
 
   function setMessageProviderPref(value) {
@@ -171,6 +172,14 @@ describe("ASRouter", () => {
         userId: "adsf",
       },
     };
+    gBrowser = {
+      selectedBrowser: {
+        constructor: { name: "MozBrowser" },
+        get ownerGlobal() {
+          return { gBrowser };
+        },
+      },
+    };
 
     ASRouterPreferences.specialConditions = {
       someCondition: true,
@@ -236,7 +245,7 @@ describe("ASRouter", () => {
       ASRouterTargeting,
       ASRouterTriggerListeners,
       QueryCache,
-      gBrowser: { selectedBrowser: {} },
+      gBrowser,
       gURLBar: {},
       isSeparateAboutWelcome: true,
       AttributionCode: fakeAttributionCode,
@@ -488,9 +497,9 @@ describe("ASRouter", () => {
       );
     });
     describe("lazily loading local test providers", () => {
-      afterEach(() => {
-        Router.uninit();
-      });
+      let justIdAndContent = ({ id, content }) => ({ id, content });
+      afterEach(() => Router.uninit());
+
       it("should add the local test providers on init if devtools are enabled", async () => {
         sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => true);
 
@@ -505,6 +514,38 @@ describe("ASRouter", () => {
 
         assert.notProperty(Router._localProviders, "PanelTestProvider");
       });
+      it("should flatten experiment translated messages from local test providers if devtools are enabled...", async () => {
+        sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => true);
+
+        await createRouterAndInit();
+
+        assert.property(Router._localProviders, "PanelTestProvider");
+
+        expect(
+          Router.state.messages.map(justIdAndContent)
+        ).to.deep.include.members([
+          { id: "experimentL10n", content: { text: "UniqueText" } },
+        ]);
+      });
+      it("...but not if devtools are disabled", async () => {
+        sandbox.stub(ASRouterPreferences, "devtoolsEnabled").get(() => false);
+
+        await createRouterAndInit();
+
+        assert.notProperty(Router._localProviders, "PanelTestProvider");
+
+        let justIdAndContentMessages =
+          Router.state.messages.map(justIdAndContent);
+        expect(justIdAndContentMessages).not.to.deep.include.members([
+          { id: "experimentL10n", content: { text: "UniqueText" } },
+        ]);
+        expect(justIdAndContentMessages).to.deep.include.members([
+          {
+            id: "experimentL10n",
+            content: { text: { $l10n: { text: "UniqueText" } } },
+          },
+        ]);
+      });
     });
   });
 
@@ -521,7 +562,7 @@ describe("ASRouter", () => {
         Router.onPrefChange
       );
     });
-    it("should send a AS_ROUTER_TARGETING_UPDATE message", async () => {
+    it("should call clearChildMessages (does nothing, see bug 1899028)", async () => {
       const messageTargeted = {
         id: "1",
         campaign: "foocampaign",
@@ -950,14 +991,13 @@ describe("ASRouter", () => {
         .rejects("fake error");
       await createRouterAndInit();
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_ERROR",
           event_context: "remotey-settingsy",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
     it("should dispatch undesired event if RemoteSettings returns no messages", async () => {
@@ -965,14 +1005,13 @@ describe("ASRouter", () => {
         .stub(MessageLoaderUtils, "_getRemoteSettingsMessages")
         .resolves([]);
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_NO_MESSAGES",
           event_context: "remotey-settingsy",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
     it("should download the attachment if RemoteSettings returns some messages", async () => {
@@ -1013,14 +1052,13 @@ describe("ASRouter", () => {
       await createRouterAndInit([provider]);
 
       assert.calledWith(initParams.dispatchCFRAction, {
+        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
         data: {
           action: "asrouter_undesired_event",
+          message_id: "n/a",
           event: "ASR_RS_NO_MESSAGES",
           event_context: "ms-language-packs",
-          message_id: "n/a",
         },
-        meta: { from: "ActivityStream:Content", to: "ActivityStream:Main" },
-        type: "AS_ROUTER_TELEMETRY_USER_EVENT",
       });
     });
   });
@@ -1591,7 +1629,7 @@ describe("ASRouter", () => {
 
       await Router.sendTriggerMessage({
         tabId: 0,
-        browser: {},
+        browser: gBrowser.selectedBrowser,
         id: "firstRun",
       });
 
@@ -1601,7 +1639,7 @@ describe("ASRouter", () => {
         {
           id: "firstRun",
           param: undefined,
-          context: undefined,
+          context: { browserIsSelected: true },
         }
       );
     });
