@@ -12,10 +12,16 @@ import { CustomizeMenu } from "content-src/components/CustomizeMenu/CustomizeMen
 import React from "react";
 import { Search } from "content-src/components/Search/Search";
 import { Sections } from "content-src/components/Sections/Sections";
+import { Logo } from "content-src/components/Logo/Logo";
 import { Weather } from "content-src/components/Weather/Weather";
+import { Notifications } from "content-src/components/Notifications/Notifications";
+import { TopicSelection } from "content-src/components/DiscoveryStreamComponents/TopicSelection/TopicSelection";
+import { WallpaperFeatureHighlight } from "../DiscoveryStreamComponents/FeatureHighlight/WallpaperFeatureHighlight";
 
 const VISIBLE = "visible";
 const VISIBILITY_CHANGE_EVENT = "visibilitychange";
+const WALLPAPER_HIGHLIGHT_DISMISSED_PREF =
+  "newtabWallpapers.highlightDismissed";
 
 export const PrefsButton = ({ onClick, icon }) => (
   <div className="prefs-button">
@@ -111,9 +117,13 @@ export class BaseContent extends React.PureComponent {
     this.handleOnKeyDown = this.handleOnKeyDown.bind(this);
     this.onWindowScroll = debounce(this.onWindowScroll.bind(this), 5);
     this.setPref = this.setPref.bind(this);
+    this.shouldShowWallpapersHighlight =
+      this.shouldShowWallpapersHighlight.bind(this);
     this.updateWallpaper = this.updateWallpaper.bind(this);
     this.prefersDarkQuery = null;
     this.handleColorModeChange = this.handleColorModeChange.bind(this);
+    this.shouldDisplayTopicSelectionModal =
+      this.shouldDisplayTopicSelectionModal.bind(this);
     this.state = {
       fixedSearch: false,
       firstVisibleTimestamp: null,
@@ -134,10 +144,12 @@ export class BaseContent extends React.PureComponent {
     global.addEventListener("keydown", this.handleOnKeyDown);
     if (this.props.document.visibilityState === VISIBLE) {
       this.setFirstVisibleTimestamp();
+      this.shouldDisplayTopicSelectionModal();
     } else {
       this._onVisibilityChange = () => {
         if (this.props.document.visibilityState === VISIBLE) {
           this.setFirstVisibleTimestamp();
+          this.shouldDisplayTopicSelectionModal();
           this.props.document.removeEventListener(
             VISIBILITY_CHANGE_EVENT,
             this._onVisibilityChange
@@ -259,17 +271,19 @@ export class BaseContent extends React.PureComponent {
 
   async updateWallpaper() {
     const prefs = this.props.Prefs.values;
+    const wallpaperLight = prefs["newtabWallpapers.wallpaper-light"];
+    const wallpaperDark = prefs["newtabWallpapers.wallpaper-dark"];
     const { wallpaperList } = this.props.Wallpapers;
 
     if (wallpaperList) {
       const lightWallpaper =
-        wallpaperList.find(
-          wp => wp.title === prefs["newtabWallpapers.wallpaper-light"]
-        ) || "";
+        wallpaperList.find(wp => wp.title === wallpaperLight) || "";
       const darkWallpaper =
-        wallpaperList.find(
-          wp => wp.title === prefs["newtabWallpapers.wallpaper-dark"]
-        ) || "";
+        wallpaperList.find(wp => wp.title === wallpaperDark) || "";
+
+      const wallpaperColor =
+        darkWallpaper?.solid_color || lightWallpaper?.solid_color || "";
+
       global.document?.body.style.setProperty(
         `--newtab-wallpaper-light`,
         `url(${lightWallpaper?.wallpaperUrl || ""})`
@@ -280,27 +294,160 @@ export class BaseContent extends React.PureComponent {
         `url(${darkWallpaper?.wallpaperUrl || ""})`
       );
 
-      // Add helper class to body if user has a wallpaper selected
-      if (lightWallpaper) {
-        global.document?.body.classList.add("hasWallpaperLight");
+      global.document?.body.style.setProperty(
+        `--newtab-wallpaper-color`,
+        wallpaperColor || "transparent"
+      );
+
+      let wallpaperTheme = "";
+
+      // If we have a solid colour set, let's see how dark it is.
+      if (wallpaperColor) {
+        const rgbColors = this.getRGBColors(wallpaperColor);
+        const isColorDark = this.isWallpaperColorDark(rgbColors);
+        wallpaperTheme = isColorDark ? "dark" : "light";
+      } else {
+        // Grab the contrast of the currently displayed wallpaper.
+        const { theme } =
+          this.state.colorMode === "light" ? lightWallpaper : darkWallpaper;
+
+        if (theme) {
+          wallpaperTheme = theme;
+        }
       }
 
-      if (darkWallpaper) {
-        global.document?.body.classList.add("hasWallpaperDark");
+      // Add helper class to body if user has a wallpaper selected
+      if (wallpaperTheme === "light") {
+        global.document?.body.classList.add("lightWallpaper");
+        global.document?.body.classList.remove("darkWallpaper");
+      }
+
+      if (wallpaperTheme === "dark") {
+        global.document?.body.classList.add("darkWallpaper");
+        global.document?.body.classList.remove("lightWallpaper");
+      }
+    }
+  }
+
+  // Contains all the logic to show the wallpapers Feature Highlight
+  shouldShowWallpapersHighlight() {
+    const prefs = this.props.Prefs.values;
+
+    // If wallpapers (or v2 wallpapers) are not enabled, don't show the highlight.
+    const wallpapersEnabled = prefs["newtabWallpapers.enabled"];
+    const wallpapersV2Enabled = prefs["newtabWallpapers.v2.enabled"];
+    if (!wallpapersEnabled || !wallpapersV2Enabled) {
+      return false;
+    }
+
+    // If user has interacted/dismissed the highlight, don't show
+    const wallpapersHighlightDismissed =
+      prefs[WALLPAPER_HIGHLIGHT_DISMISSED_PREF];
+    if (wallpapersHighlightDismissed) {
+      return false;
+    }
+
+    // If the user has selected a wallpaper, don't show the pop-up
+    const activeWallpaperLight = prefs[`newtabWallpapers.wallpaper-light`];
+    const activeWallpaperDark = prefs[`newtabWallpapers.wallpaper-dark`];
+    if (activeWallpaperLight || activeWallpaperDark) {
+      this.props.dispatch(ac.SetPref(WALLPAPER_HIGHLIGHT_DISMISSED_PREF, true));
+      return false;
+    }
+
+    // If the user has seen* the highlight more than three times
+    // *Seen means they loaded HNT page and the highlight was observed for more than 3 seconds
+    const { highlightSeenCounter } = this.props.Wallpapers;
+    if (highlightSeenCounter.value > 3) {
+      return false;
+    }
+
+    // Show the highlight if available
+    const wallpapersHighlightEnabled =
+      prefs["newtabWallpapers.highlightEnabled"];
+    if (wallpapersHighlightEnabled) {
+      return true;
+    }
+
+    // Default return value
+    return false;
+  }
+
+  getRGBColors(input) {
+    if (input.length !== 7) {
+      return [];
+    }
+
+    const r = parseInt(input.substr(1, 2), 16);
+    const g = parseInt(input.substr(3, 2), 16);
+    const b = parseInt(input.substr(5, 2), 16);
+
+    return [r, g, b];
+  }
+
+  isWallpaperColorDark([r, g, b]) {
+    return 0.2125 * r + 0.7154 * g + 0.0721 * b <= 110;
+  }
+
+  shouldDisplayTopicSelectionModal() {
+    const prefs = this.props.Prefs.values;
+    const pocketEnabled =
+      prefs["feeds.section.topstories"] && prefs["feeds.system.topstories"];
+    const topicSelectionOnboardingEnabled =
+      prefs["discoverystream.topicSelection.onboarding.enabled"] &&
+      pocketEnabled;
+    const maybeShowModal =
+      prefs["discoverystream.topicSelection.onboarding.maybeDisplay"];
+    const displayTimeout =
+      prefs["discoverystream.topicSelection.onboarding.displayTimeout"];
+    const lastDisplayed =
+      prefs["discoverystream.topicSelection.onboarding.lastDisplayed"];
+    const displayCount =
+      prefs["discoverystream.topicSelection.onboarding.displayCount"];
+
+    if (
+      !maybeShowModal ||
+      !prefs["discoverystream.topicSelection.enabled"] ||
+      !topicSelectionOnboardingEnabled
+    ) {
+      return;
+    }
+
+    const day = 24 * 60 * 60 * 1000;
+    const now = new Date().getTime();
+
+    const timeoutOccured = now - parseFloat(lastDisplayed) > displayTimeout;
+    if (displayCount < 3) {
+      if (displayCount === 0 || timeoutOccured) {
+        this.props.dispatch(
+          ac.BroadcastToContent({ type: at.TOPIC_SELECTION_SPOTLIGHT_OPEN })
+        );
+        this.setPref(
+          "discoverystream.topicSelection.onboarding.displayTimeout",
+          day
+        );
       }
     }
   }
 
   render() {
     const { props } = this;
-    const { App } = props;
+    const { App, DiscoveryStream } = props;
     const { initialized, customizeMenuVisible } = App;
     const prefs = props.Prefs.values;
 
+    const layoutsVariantAEnabled = prefs["newtabLayouts.variant-a"];
+    const layoutsVariantBEnabled = prefs["newtabLayouts.variant-b"];
+    const layoutsVariantAorB = layoutsVariantAEnabled || layoutsVariantBEnabled;
+
     const activeWallpaper =
       prefs[`newtabWallpapers.wallpaper-${this.state.colorMode}`];
     const wallpapersEnabled = prefs["newtabWallpapers.enabled"];
+    const wallpapersV2Enabled = prefs["newtabWallpapers.v2.enabled"];
     const weatherEnabled = prefs.showWeather;
+    const { showTopicSelection } = DiscoveryStream;
+    const mayShowTopicSelection =
+      showTopicSelection && prefs["discoverystream.topicSelection.enabled"];
 
     const { pocketConfig } = prefs;
 
@@ -340,6 +487,21 @@ export class BaseContent extends React.PureComponent {
     const mayHaveSponsoredStories = prefs["system.showSponsored"];
     const mayHaveWeather = prefs["system.showWeather"];
     const { mayHaveSponsoredTopSites } = prefs;
+    const supportUrl = prefs["support.url"];
+
+    const hasThumbsUpDownLayout =
+      prefs["discoverystream.thumbsUpDown.searchTopsitesCompact"];
+    const hasThumbsUpDown = prefs["discoverystream.thumbsUpDown.enabled"];
+
+    const featureClassName = [
+      weatherEnabled && mayHaveWeather && "has-weather", // Show is weather is enabled/visible
+      prefs.showSearch ? "has-search" : "no-search",
+      layoutsVariantAEnabled ? "layout-variant-a" : "", // Layout experiment variant A
+      layoutsVariantBEnabled ? "layout-variant-b" : "", // Layout experiment variant B
+      pocketEnabled ? "has-recommended-stories" : "no-recommended-stories",
+    ]
+      .filter(v => v)
+      .join(" ");
 
     const outerClassName = [
       "outer-wrapper",
@@ -350,31 +512,54 @@ export class BaseContent extends React.PureComponent {
         !noSectionsEnabled &&
         "fixed-search",
       prefs.showSearch && noSectionsEnabled && "only-search",
+      prefs["feeds.topsites"] &&
+        !pocketEnabled &&
+        !prefs.showSearch &&
+        "only-topsites",
+      noSectionsEnabled && "no-sections",
       prefs["logowordmark.alwaysVisible"] && "visible-logo",
+      hasThumbsUpDownLayout && hasThumbsUpDown && "thumbs-ui-compact",
     ]
       .filter(v => v)
       .join(" ");
-    if (wallpapersEnabled) {
+    if (wallpapersEnabled || wallpapersV2Enabled) {
       this.updateWallpaper();
     }
 
     return (
-      <div>
-        <CustomizeMenu
-          onClose={this.closeCustomizationMenu}
-          onOpen={this.openCustomizationMenu}
-          openPreferences={this.openPreferences}
-          setPref={this.setPref}
-          enabledSections={enabledSections}
-          wallpapersEnabled={wallpapersEnabled}
-          activeWallpaper={activeWallpaper}
-          pocketRegion={pocketRegion}
-          mayHaveSponsoredTopSites={mayHaveSponsoredTopSites}
-          mayHaveSponsoredStories={mayHaveSponsoredStories}
-          mayHaveWeather={mayHaveWeather}
-          spocMessageVariant={spocMessageVariant}
-          showing={customizeMenuVisible}
-        />
+      <div className={featureClassName}>
+        {/* Floating menu for customize menu toggle */}
+        <menu className="personalizeButtonWrapper">
+          <CustomizeMenu
+            onClose={this.closeCustomizationMenu}
+            onOpen={this.openCustomizationMenu}
+            openPreferences={this.openPreferences}
+            setPref={this.setPref}
+            enabledSections={enabledSections}
+            wallpapersEnabled={wallpapersEnabled}
+            wallpapersV2Enabled={wallpapersV2Enabled}
+            activeWallpaper={activeWallpaper}
+            pocketRegion={pocketRegion}
+            mayHaveSponsoredTopSites={mayHaveSponsoredTopSites}
+            mayHaveSponsoredStories={mayHaveSponsoredStories}
+            mayHaveWeather={mayHaveWeather}
+            spocMessageVariant={spocMessageVariant}
+            showing={customizeMenuVisible}
+          />
+          {this.shouldShowWallpapersHighlight() && (
+            <WallpaperFeatureHighlight
+              position="inset-block-end inset-inline-start"
+              dispatch={this.props.dispatch}
+            />
+          )}
+        </menu>
+        <div className="weatherWrapper">
+          {weatherEnabled && (
+            <ErrorBoundary>
+              <Weather />
+            </ErrorBoundary>
+          )}
+        </div>
         {/* eslint-disable-next-line jsx-a11y/click-events-have-key-events, jsx-a11y/no-static-element-interactions*/}
         <div className={outerClassName} onClick={this.closeCustomizationMenu}>
           <main>
@@ -391,6 +576,10 @@ export class BaseContent extends React.PureComponent {
                 </ErrorBoundary>
               </div>
             )}
+            {/* Bug 1914055: Show logo regardless if search is enabled */}
+            {!prefs.showSearch && layoutsVariantAorB && !noSectionsEnabled && (
+              <Logo />
+            )}
             <div className={`body-wrapper${initialized ? " on" : ""}`}>
               {isDiscoveryStream ? (
                 <ErrorBoundary className="borderless-error">
@@ -408,12 +597,16 @@ export class BaseContent extends React.PureComponent {
             {wallpapersEnabled && this.renderWallpaperAttribution()}
           </main>
           <aside>
-            {weatherEnabled && (
+            {this.props.Notifications?.showNotifications && (
               <ErrorBoundary>
-                <Weather />
+                <Notifications dispatch={this.props.dispatch} />
               </ErrorBoundary>
             )}
           </aside>
+          {/* Only show the modal on currently visible pages (not preloaded) */}
+          {mayShowTopicSelection && pocketEnabled && (
+            <TopicSelection supportUrl={supportUrl} />
+          )}
         </div>
       </div>
     );
@@ -429,6 +622,7 @@ export const Base = connect(state => ({
   Prefs: state.Prefs,
   Sections: state.Sections,
   DiscoveryStream: state.DiscoveryStream,
+  Notifications: state.Notifications,
   Search: state.Search,
   Wallpapers: state.Wallpapers,
   Weather: state.Weather,
