@@ -9,7 +9,6 @@ const {
   Personalization,
   DiscoveryStream,
   Search,
-  ASRouter,
 } = reducers;
 import { actionTypes as at } from "common/Actions.mjs";
 
@@ -1518,8 +1517,4 @@ describe("Reducers", () => {
       assert.propertyVal(nextState, "disable", false);
     });
   });
-  it("should set initialized to true on AS_ROUTER_INITIALIZED", () => {
-    const nextState = ASRouter(undefined, { type: "AS_ROUTER_INITIALIZED" });
-    assert.propertyVal(nextState, "initialized", true);
-  });
 });
