@@ -6,6 +6,7 @@
 #define UNICODE
 
 #include "nsWindowsShellService.h"
+#include "nsWindowsShellServiceInternal.h"
 
 #include "BinaryPath.h"
 #include "imgIContainer.h"
@@ -19,6 +20,7 @@
 #include "nsIStringBundle.h"
 #include "nsIMIMEService.h"
 #include "nsNetUtil.h"
+#include "nsProxyRelease.h"
 #include "nsServiceManagerUtils.h"
 #include "nsShellService.h"
 #include "nsDirectoryServiceUtils.h"
@@ -26,6 +28,7 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsIWindowsRegKey.h"
 #include "nsUnicharUtils.h"
+#include "nsWindowsHelpers.h"
 #include "nsXULAppAPI.h"
 #include "mozilla/WindowsVersion.h"
 #include "mozilla/dom/Element.h"
@@ -54,6 +57,21 @@ PSSTDAPI PropVariantToString(REFPROPVARIANT propvar, PWSTR psz, UINT cch);
 #  define UNLEN 256
 #else
 #  include <Lmcons.h>  // For UNLEN
+#  include <wrl.h>
+#  include <wrl/wrappers/corewrappers.h>
+#  include <windows.applicationmodel.h>
+#  include <windows.applicationmodel.core.h>
+#  include <windows.applicationmodel.activation.h>
+#  include <windows.foundation.h>
+#  include <windows.ui.startscreen.h>
+using namespace Microsoft::WRL;
+using namespace ABI::Windows;
+using namespace ABI::Windows::Foundation;
+using namespace ABI::Windows::Foundation::Collections;
+using namespace ABI::Windows::ApplicationModel;
+using namespace ABI::Windows::ApplicationModel::Core;
+using namespace ABI::Windows::UI::StartScreen;
+using namespace Microsoft::WRL::Wrappers;
 #endif
 
 #include <comutil.h>
@@ -746,19 +764,12 @@ static nsresult WriteShortcutToLog(nsIFile* aShortcutsLogDir,
   return NS_OK;
 }
 
-static nsresult CreateShortcutImpl(
-    nsIFile* aBinary, const CopyableTArray<nsString>& aArguments,
-    const nsAString& aDescription, nsIFile* aIconFile, uint16_t aIconIndex,
-    const nsAString& aAppUserModelId, KNOWNFOLDERID aShortcutFolder,
-    const nsAString& aShortcutName, const nsString& aShortcutFile,
-    nsIFile* aShortcutsLogDir) {
-  NS_ENSURE_ARG(aBinary);
-  NS_ENSURE_ARG(aIconFile);
-
-  nsresult rv =
-      WriteShortcutToLog(aShortcutsLogDir, aShortcutFolder, aShortcutName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
+nsresult CreateShellLinkObject(nsIFile* aBinary,
+                               const CopyableTArray<nsString>& aArguments,
+                               const nsAString& aDescription,
+                               nsIFile* aIconFile, uint16_t aIconIndex,
+                               const nsAString& aAppUserModelId,
+                               IShellLinkW** aLink) {
   RefPtr<IShellLinkW> link;
   HRESULT hr = CoCreateInstance(CLSID_ShellLink, nullptr, CLSCTX_INPROC_SERVER,
                                 IID_IShellLinkW, getter_AddRefs(link));
@@ -778,8 +789,8 @@ static nsresult CreateShortcutImpl(
 
   // TODO: Properly escape quotes in the string, see bug 1604287.
   nsString arguments;
-  for (auto& arg : aArguments) {
-    arguments.AppendPrintf("\"%S\" ", static_cast<const wchar_t*>(arg.get()));
+  for (const auto& arg : aArguments) {
+    arguments += u"\""_ns + arg + u"\" "_ns;
   }
 
   link->SetArguments(arguments.get());
@@ -808,8 +819,30 @@ static nsresult CreateShortcutImpl(
     NS_ENSURE_HRESULT(hr, NS_ERROR_FAILURE);
   }
 
+  link.forget(aLink);
+  return NS_OK;
+}
+
+static nsresult CreateShortcutImpl(
+    nsIFile* aBinary, const CopyableTArray<nsString>& aArguments,
+    const nsAString& aDescription, nsIFile* aIconFile, uint16_t aIconIndex,
+    const nsAString& aAppUserModelId, KNOWNFOLDERID aShortcutFolder,
+    const nsAString& aShortcutName, const nsString& aShortcutFile,
+    nsIFile* aShortcutsLogDir) {
+  NS_ENSURE_ARG(aBinary);
+  NS_ENSURE_ARG(aIconFile);
+
+  nsresult rv =
+      WriteShortcutToLog(aShortcutsLogDir, aShortcutFolder, aShortcutName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  RefPtr<IShellLinkW> link;
+  rv = CreateShellLinkObject(aBinary, aArguments, aDescription, aIconFile,
+                             aIconIndex, aAppUserModelId, getter_AddRefs(link));
+  NS_ENSURE_SUCCESS(rv, rv);
+
   RefPtr<IPersistFile> persist;
-  hr = link->QueryInterface(IID_IPersistFile, getter_AddRefs(persist));
+  HRESULT hr = link->QueryInterface(IID_IPersistFile, getter_AddRefs(persist));
   NS_ENSURE_HRESULT(hr, NS_ERROR_FAILURE);
 
   hr = persist->Save(aShortcutFile.get(), TRUE);
@@ -1251,6 +1284,33 @@ NS_IMETHODIMP nsWindowsShellService::HasMatchingShortcut(
 }
 
 static bool IsCurrentAppPinnedToTaskbarSync(const nsAString& aumid) {
+  // Use new Windows pinning APIs to determine whether or not we're pinned.
+  // If these fail we can safely fall back to the old method for regular
+  // installs however MSIX will always return false.
+
+  // Bug 1911343: Add a check for whether we're looking for a regular pin
+  // or PB pin based on the AUMID value once private browser pinning
+  // is supported on MSIX.
+  // Right now only run this check on MSIX to avoid
+  // false positives when only private browsing is pinned.
+  if (widget::WinUtils::HasPackageIdentity()) {
+    auto pinWithWin11TaskbarAPIResults =
+        IsCurrentAppPinnedToTaskbarWin11(false);
+    switch (pinWithWin11TaskbarAPIResults.result) {
+      case Win11PinToTaskBarResultStatus::NotPinned:
+        return false;
+        break;
+      case Win11PinToTaskBarResultStatus::AlreadyPinned:
+        return true;
+        break;
+      default:
+        // Fall through to the old mechanism.
+        // The old mechanism should continue working for non-MSIX
+        // builds.
+        break;
+    }
+  }
+
   // There are two shortcut targets that we created. One always matches the
   // binary we're running as (eg: firefox.exe). The other is the wrapper
   // for launching in Private Browsing mode. We need to inspect shortcuts
@@ -1697,7 +1757,7 @@ static nsresult PinCurrentAppToTaskbarImpl(
   }
 
   auto pinWithWin11TaskbarAPIResults =
-      PinCurrentAppToTaskbarWin11(aCheckOnly, aAppUserModelId, shortcutPath);
+      PinCurrentAppToTaskbarWin11(aCheckOnly, aAppUserModelId);
   switch (pinWithWin11TaskbarAPIResults.result) {
     case Win11PinToTaskBarResultStatus::NotSupported:
       // Fall through to the win 10 mechanism
@@ -1707,6 +1767,7 @@ static nsresult PinCurrentAppToTaskbarImpl(
     case Win11PinToTaskBarResultStatus::AlreadyPinned:
       return NS_OK;
 
+    case Win11PinToTaskBarResultStatus::NotPinned:
     case Win11PinToTaskBarResultStatus::NotCurrentlyAllowed:
     case Win11PinToTaskBarResultStatus::Failed:
       // return NS_ERROR_FAILURE;
@@ -1831,12 +1892,6 @@ NS_IMETHODIMP
 nsWindowsShellService::PinCurrentAppToTaskbarAsync(bool aPrivateBrowsing,
                                                    JSContext* aCx,
                                                    dom::Promise** aPromise) {
-  // https://bugzilla.mozilla.org/show_bug.cgi?id=1712628 tracks implementing
-  // this for MSIX packages.
-  if (widget::WinUtils::HasPackageIdentity()) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-
   return PinCurrentAppToTaskbarAsyncImpl(
       /* aCheckOnly */ false, aPrivateBrowsing, aCx, aPromise);
 }
@@ -1844,12 +1899,6 @@ nsWindowsShellService::PinCurrentAppToTaskbarAsync(bool aPrivateBrowsing,
 NS_IMETHODIMP
 nsWindowsShellService::CheckPinCurrentAppToTaskbarAsync(
     bool aPrivateBrowsing, JSContext* aCx, dom::Promise** aPromise) {
-  // https://bugzilla.mozilla.org/show_bug.cgi?id=1712628 tracks implementing
-  // this for MSIX packages.
-  if (widget::WinUtils::HasPackageIdentity()) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-
   return PinCurrentAppToTaskbarAsyncImpl(
       /* aCheckOnly = */ true, aPrivateBrowsing, aCx, aPromise);
 }
@@ -1857,12 +1906,6 @@ nsWindowsShellService::CheckPinCurrentAppToTaskbarAsync(
 NS_IMETHODIMP
 nsWindowsShellService::IsCurrentAppPinnedToTaskbarAsync(
     const nsAString& aumid, JSContext* aCx, /* out */ dom::Promise** aPromise) {
-  // https://bugzilla.mozilla.org/show_bug.cgi?id=1712628 tracks implementing
-  // this for MSIX packages.
-  if (widget::WinUtils::HasPackageIdentity()) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-
   if (!NS_IsMainThread()) {
     return NS_ERROR_NOT_SAME_THREAD;
   }
@@ -1907,6 +1950,612 @@ nsWindowsShellService::IsCurrentAppPinnedToTaskbarAsync(
   return NS_OK;
 }
 
+#ifndef __MINGW32__
+#  define RESOLVE_AND_RETURN(HOLDER, RESOLVE, RETURN)                \
+    NS_DispatchToMainThread(NS_NewRunnableFunction(                  \
+        __func__, [resolveVal = (RESOLVE), promiseHolder = HOLDER] { \
+          promiseHolder.get()->get()->MaybeResolve(resolveVal);      \
+        }));                                                         \
+    return RETURN
+
+#  define REJECT_AND_RETURN(HOLDER, REJECT, RETURN)                 \
+    NS_DispatchToMainThread(                                        \
+        NS_NewRunnableFunction(__func__, [promiseHolder = HOLDER] { \
+          promiseHolder.get()->get()->MaybeReject(REJECT);          \
+        }));                                                        \
+    return RETURN
+
+static void EnableLaunchOnLoginMSIXAsyncImpl(
+    const nsString& capturedTaskId,
+    const RefPtr<nsMainThreadPtrHolder<dom::Promise>> promiseHolder) {
+  ComPtr<IStartupTaskStatics> startupTaskStatics;
+  HRESULT hr = GetActivationFactory(
+      HStringReference(RuntimeClass_Windows_ApplicationModel_StartupTask).Get(),
+      &startupTaskStatics);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  ComPtr<IAsyncOperation<StartupTask*>> getTaskOperation = nullptr;
+  hr = startupTaskStatics->GetAsync(
+      HStringReference(capturedTaskId.get()).Get(), &getTaskOperation);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  auto getTaskCallback =
+      Callback<IAsyncOperationCompletedHandler<StartupTask*>>(
+          [promiseHolder](IAsyncOperation<StartupTask*>* operation,
+                          AsyncStatus status) -> HRESULT {
+            if (status != AsyncStatus::Completed) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IStartupTask> startupTask;
+            HRESULT hr = operation->GetResults(&startupTask);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IAsyncOperation<StartupTaskState>> enableOperation;
+            hr = startupTask->RequestEnableAsync(&enableOperation);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            // Set another callback for enabling the startup task
+            auto enableHandler =
+                Callback<IAsyncOperationCompletedHandler<StartupTaskState>>(
+                    [promiseHolder](
+                        IAsyncOperation<StartupTaskState>* operation,
+                        AsyncStatus status) -> HRESULT {
+                      StartupTaskState resultState;
+                      HRESULT hr = operation->GetResults(&resultState);
+                      if (SUCCEEDED(hr) && status == AsyncStatus::Completed) {
+                        RESOLVE_AND_RETURN(promiseHolder, true, S_OK);
+                      }
+                      RESOLVE_AND_RETURN(promiseHolder, false, S_OK);
+                    });
+            hr = enableOperation->put_Completed(enableHandler.Get());
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, hr);
+            }
+            return hr;
+          });
+  hr = getTaskOperation->put_Completed(getTaskCallback.Get());
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+}
+
+static void DisableLaunchOnLoginMSIXAsyncImpl(
+    const nsString& capturedTaskId,
+    const RefPtr<nsMainThreadPtrHolder<dom::Promise>> promiseHolder) {
+  ComPtr<IStartupTaskStatics> startupTaskStatics;
+  HRESULT hr = GetActivationFactory(
+      HStringReference(RuntimeClass_Windows_ApplicationModel_StartupTask).Get(),
+      &startupTaskStatics);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  ComPtr<IAsyncOperation<StartupTask*>> getTaskOperation = nullptr;
+  hr = startupTaskStatics->GetAsync(
+      HStringReference(capturedTaskId.get()).Get(), &getTaskOperation);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  auto getTaskCallback =
+      Callback<IAsyncOperationCompletedHandler<StartupTask*>>(
+          [promiseHolder](IAsyncOperation<StartupTask*>* operation,
+                          AsyncStatus status) -> HRESULT {
+            if (status != AsyncStatus::Completed) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IStartupTask> startupTask;
+            HRESULT hr = operation->GetResults(&startupTask);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            hr = startupTask->Disable();
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            RESOLVE_AND_RETURN(promiseHolder, true, S_OK);
+          });
+  hr = getTaskOperation->put_Completed(getTaskCallback.Get());
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+}
+
+static void GetLaunchOnLoginEnabledMSIXAsyncImpl(
+    const nsString& capturedTaskId,
+    const RefPtr<nsMainThreadPtrHolder<dom::Promise>> promiseHolder) {
+  ComPtr<IStartupTaskStatics> startupTaskStatics;
+  HRESULT hr = GetActivationFactory(
+      HStringReference(RuntimeClass_Windows_ApplicationModel_StartupTask).Get(),
+      &startupTaskStatics);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  ComPtr<IAsyncOperation<StartupTask*>> getTaskOperation = nullptr;
+  hr = startupTaskStatics->GetAsync(
+      HStringReference(capturedTaskId.get()).Get(), &getTaskOperation);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  auto getTaskCallback =
+      Callback<IAsyncOperationCompletedHandler<StartupTask*>>(
+          [promiseHolder](IAsyncOperation<StartupTask*>* operation,
+                          AsyncStatus status) -> HRESULT {
+            if (status != AsyncStatus::Completed) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IStartupTask> startupTask;
+            HRESULT hr = operation->GetResults(&startupTask);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            StartupTaskState state;
+            hr = startupTask->get_State(&state);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            switch (state) {
+              case StartupTaskState_EnabledByPolicy:
+                RESOLVE_AND_RETURN(
+                    promiseHolder,
+                    nsIWindowsShellService::LaunchOnLoginEnabledEnumerator::
+                        LAUNCH_ON_LOGIN_ENABLED_BY_POLICY,
+                    S_OK);
+                break;
+              case StartupTaskState_Enabled:
+                RESOLVE_AND_RETURN(
+                    promiseHolder,
+                    nsIWindowsShellService::LaunchOnLoginEnabledEnumerator::
+                        LAUNCH_ON_LOGIN_ENABLED,
+                    S_OK);
+                break;
+              case StartupTaskState_DisabledByUser:
+              case StartupTaskState_DisabledByPolicy:
+                RESOLVE_AND_RETURN(
+                    promiseHolder,
+                    nsIWindowsShellService::LaunchOnLoginEnabledEnumerator::
+                        LAUNCH_ON_LOGIN_DISABLED_BY_SETTINGS,
+                    S_OK);
+                break;
+              default:
+                RESOLVE_AND_RETURN(
+                    promiseHolder,
+                    nsIWindowsShellService::LaunchOnLoginEnabledEnumerator::
+                        LAUNCH_ON_LOGIN_DISABLED,
+                    S_OK);
+            }
+          });
+  hr = getTaskOperation->put_Completed(getTaskCallback.Get());
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::EnableLaunchOnLoginMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  if (!widget::WinUtils::HasPackageIdentity()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  if (!NS_IsMainThread()) {
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
+  ErrorResult rv;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aCx), rv);
+  if (MOZ_UNLIKELY(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // A holder to pass the promise through the background task and back to
+  // the main thread when finished.
+  auto promiseHolder = MakeRefPtr<nsMainThreadPtrHolder<dom::Promise>>(
+      "EnableLaunchOnLoginMSIXAsync promise", promise);
+
+  NS_DispatchBackgroundTask(NS_NewRunnableFunction(
+      "EnableLaunchOnLoginMSIXAsync",
+      [taskId = nsString(aTaskId), promiseHolder] {
+        EnableLaunchOnLoginMSIXAsyncImpl(taskId, promiseHolder);
+      }));
+
+  promise.forget(aPromise);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::DisableLaunchOnLoginMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  if (!widget::WinUtils::HasPackageIdentity()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  if (!NS_IsMainThread()) {
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
+  ErrorResult rv;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aCx), rv);
+  if (MOZ_UNLIKELY(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // A holder to pass the promise through the background task and back to
+  // the main thread when finished.
+  auto promiseHolder = MakeRefPtr<nsMainThreadPtrHolder<dom::Promise>>(
+      "DisableLaunchOnLoginMSIXAsync promise", promise);
+
+  NS_DispatchBackgroundTask(NS_NewRunnableFunction(
+      "DisableLaunchOnLoginMSIXAsync",
+      [taskId = nsString(aTaskId), promiseHolder] {
+        DisableLaunchOnLoginMSIXAsyncImpl(taskId, promiseHolder);
+      }));
+
+  promise.forget(aPromise);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::GetLaunchOnLoginEnabledMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  if (!widget::WinUtils::HasPackageIdentity()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  if (!NS_IsMainThread()) {
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
+  ErrorResult rv;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aCx), rv);
+  if (MOZ_UNLIKELY(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // A holder to pass the promise through the background task and back to
+  // the main thread when finished.
+  auto promiseHolder = MakeRefPtr<nsMainThreadPtrHolder<dom::Promise>>(
+      "GetLaunchOnLoginEnabledMSIXAsync promise", promise);
+
+  NS_DispatchBackgroundTask(NS_NewRunnableFunction(
+      "GetLaunchOnLoginEnabledMSIXAsync",
+      [taskId = nsString(aTaskId), promiseHolder] {
+        GetLaunchOnLoginEnabledMSIXAsyncImpl(taskId, promiseHolder);
+      }));
+
+  promise.forget(aPromise);
+  return NS_OK;
+}
+
+static HRESULT GetPackage3(ComPtr<IPackage3>& package3) {
+  // Get the current package and cast it to IPackage3 so we can
+  // check for AppListEntries
+  ComPtr<IPackageStatics> packageStatics;
+  HRESULT hr = GetActivationFactory(
+      HStringReference(RuntimeClass_Windows_ApplicationModel_Package).Get(),
+      &packageStatics);
+
+  if (FAILED(hr)) {
+    return hr;
+  }
+  ComPtr<IPackage> package;
+  hr = packageStatics->get_Current(&package);
+  if (FAILED(hr)) {
+    return hr;
+  }
+  hr = package.As(&package3);
+  return hr;
+}
+
+static HRESULT GetStartScreenManager(
+    ComPtr<IVectorView<AppListEntry*>>& appListEntries,
+    ComPtr<IAppListEntry>& entry,
+    ComPtr<IStartScreenManager>& startScreenManager) {
+  unsigned int numEntries = 0;
+  HRESULT hr = appListEntries->get_Size(&numEntries);
+  if (FAILED(hr) || numEntries == 0) {
+    return E_FAIL;
+  }
+  // There's only one AppListEntry in the Firefox package and by
+  // convention our main executable should be the first in the
+  // list.
+  hr = appListEntries->GetAt(0, &entry);
+
+  // Create and init a StartScreenManager and check if we're already
+  // pinned.
+  ComPtr<IStartScreenManagerStatics> startScreenManagerStatics;
+  hr = GetActivationFactory(
+      HStringReference(RuntimeClass_Windows_UI_StartScreen_StartScreenManager)
+          .Get(),
+      &startScreenManagerStatics);
+  if (FAILED(hr)) {
+    return hr;
+  }
+
+  hr = startScreenManagerStatics->GetDefault(&startScreenManager);
+  return hr;
+}
+
+static void PinCurrentAppToStartMenuAsyncImpl(
+    bool aCheckOnly,
+    const RefPtr<nsMainThreadPtrHolder<dom::Promise>> promiseHolder) {
+  ComPtr<IPackage3> package3;
+  HRESULT hr = GetPackage3(package3);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+
+  // Get the AppList entries
+  ComPtr<IVectorView<AppListEntry*>> appListEntries;
+  ComPtr<IAsyncOperation<IVectorView<AppListEntry*>*>>
+      getAppListEntriesOperation;
+  hr = package3->GetAppListEntriesAsync(&getAppListEntriesOperation);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  auto getAppListEntriesCallback =
+      Callback<IAsyncOperationCompletedHandler<IVectorView<AppListEntry*>*>>(
+          [promiseHolder, aCheckOnly](
+              IAsyncOperation<IVectorView<AppListEntry*>*>* operation,
+              AsyncStatus status) -> HRESULT {
+            if (status != AsyncStatus::Completed) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IVectorView<AppListEntry*>> appListEntries;
+            HRESULT hr = operation->GetResults(&appListEntries);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IStartScreenManager> startScreenManager;
+            ComPtr<IAppListEntry> entry;
+            hr = GetStartScreenManager(appListEntries, entry,
+                                       startScreenManager);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IAsyncOperation<bool>> getPinnedOperation;
+            hr = startScreenManager->ContainsAppListEntryAsync(
+                entry.Get(), &getPinnedOperation);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            auto getPinnedCallback =
+                Callback<IAsyncOperationCompletedHandler<bool>>(
+                    [promiseHolder, entry, startScreenManager, aCheckOnly](
+                        IAsyncOperation<bool>* operation,
+                        AsyncStatus status) -> HRESULT {
+                      if (status != AsyncStatus::Completed) {
+                        REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE,
+                                          E_FAIL);
+                      }
+                      boolean isAlreadyPinned;
+                      HRESULT hr = operation->GetResults(&isAlreadyPinned);
+                      if (FAILED(hr)) {
+                        REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE,
+                                          E_FAIL);
+                      }
+                      // If we're already pinned we can return early
+                      // Ditto if we're just checking whether we *can* pin
+                      if (isAlreadyPinned || aCheckOnly) {
+                        RESOLVE_AND_RETURN(promiseHolder, true, S_OK);
+                      }
+                      ComPtr<IAsyncOperation<bool>> pinOperation;
+                      startScreenManager->RequestAddAppListEntryAsync(
+                          entry.Get(), &pinOperation);
+                      // Set another callback for pinning to the start menu
+                      auto pinOperationCallback =
+                          Callback<IAsyncOperationCompletedHandler<bool>>(
+                              [promiseHolder](IAsyncOperation<bool>* operation,
+                                              AsyncStatus status) -> HRESULT {
+                                if (status != AsyncStatus::Completed) {
+                                  REJECT_AND_RETURN(promiseHolder,
+                                                    NS_ERROR_FAILURE, E_FAIL);
+                                };
+                                boolean pinSuccess;
+                                HRESULT hr = operation->GetResults(&pinSuccess);
+                                if (FAILED(hr)) {
+                                  REJECT_AND_RETURN(promiseHolder,
+                                                    NS_ERROR_FAILURE, E_FAIL);
+                                }
+                                RESOLVE_AND_RETURN(promiseHolder,
+                                                   pinSuccess ? true : false,
+                                                   S_OK);
+                              });
+                      hr = pinOperation->put_Completed(
+                          pinOperationCallback.Get());
+                      if (FAILED(hr)) {
+                        REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, hr);
+                      }
+                      return hr;
+                    });
+            hr = getPinnedOperation->put_Completed(getPinnedCallback.Get());
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, hr);
+            }
+            return hr;
+          });
+  hr = getAppListEntriesOperation->put_Completed(
+      getAppListEntriesCallback.Get());
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::PinCurrentAppToStartMenuAsync(bool aCheckOnly,
+                                                     JSContext* aCx,
+                                                     dom::Promise** aPromise) {
+  if (!NS_IsMainThread()) {
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
+  // Unfortunately pinning to the Start Menu requires IAppListEntry
+  // which is only implemented for packaged applications.
+  if (!widget::WinUtils::HasPackageIdentity()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  ErrorResult rv;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aCx), rv);
+  if (MOZ_UNLIKELY(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // A holder to pass the promise through the background task and back to
+  // the main thread when finished.
+  auto promiseHolder = MakeRefPtr<nsMainThreadPtrHolder<dom::Promise>>(
+      "PinCurrentAppToStartMenuAsync promise", promise);
+  NS_DispatchBackgroundTask(NS_NewRunnableFunction(
+      "PinCurrentAppToStartMenuAsync", [aCheckOnly, promiseHolder] {
+        PinCurrentAppToStartMenuAsyncImpl(aCheckOnly, promiseHolder);
+      }));
+  promise.forget(aPromise);
+  return NS_OK;
+}
+
+static void IsCurrentAppPinnedToStartMenuAsyncImpl(
+    const RefPtr<nsMainThreadPtrHolder<dom::Promise>> promiseHolder) {
+  ComPtr<IPackage3> package3;
+  HRESULT hr = GetPackage3(package3);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+
+  // Get the AppList entries
+  ComPtr<IVectorView<AppListEntry*>> appListEntries;
+  ComPtr<IAsyncOperation<IVectorView<AppListEntry*>*>>
+      getAppListEntriesOperation;
+  hr = package3->GetAppListEntriesAsync(&getAppListEntriesOperation);
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+  auto getAppListEntriesCallback =
+      Callback<IAsyncOperationCompletedHandler<IVectorView<AppListEntry*>*>>(
+          [promiseHolder](
+              IAsyncOperation<IVectorView<AppListEntry*>*>* operation,
+              AsyncStatus status) -> HRESULT {
+            if (status != AsyncStatus::Completed) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IVectorView<AppListEntry*>> appListEntries;
+            HRESULT hr = operation->GetResults(&appListEntries);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IStartScreenManager> startScreenManager;
+            ComPtr<IAppListEntry> entry;
+            hr = GetStartScreenManager(appListEntries, entry,
+                                       startScreenManager);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            ComPtr<IAsyncOperation<bool>> getPinnedOperation;
+            hr = startScreenManager->ContainsAppListEntryAsync(
+                entry.Get(), &getPinnedOperation);
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, E_FAIL);
+            }
+            auto getPinnedCallback =
+                Callback<IAsyncOperationCompletedHandler<bool>>(
+                    [promiseHolder, entry, startScreenManager](
+                        IAsyncOperation<bool>* operation,
+                        AsyncStatus status) -> HRESULT {
+                      if (status != AsyncStatus::Completed) {
+                        REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE,
+                                          E_FAIL);
+                      }
+                      boolean isAlreadyPinned;
+                      HRESULT hr = operation->GetResults(&isAlreadyPinned);
+                      if (FAILED(hr)) {
+                        REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE,
+                                          E_FAIL);
+                      }
+                      RESOLVE_AND_RETURN(promiseHolder,
+                                         isAlreadyPinned ? true : false, S_OK);
+                    });
+            hr = getPinnedOperation->put_Completed(getPinnedCallback.Get());
+            if (FAILED(hr)) {
+              REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, hr);
+            }
+            return hr;
+          });
+  hr = getAppListEntriesOperation->put_Completed(
+      getAppListEntriesCallback.Get());
+  if (FAILED(hr)) {
+    REJECT_AND_RETURN(promiseHolder, NS_ERROR_FAILURE, /* void */);
+  }
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::IsCurrentAppPinnedToStartMenuAsync(
+    JSContext* aCx, dom::Promise** aPromise) {
+  if (!NS_IsMainThread()) {
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
+  // Unfortunately pinning to the Start Menu requires IAppListEntry
+  // which is only implemented for packaged applications.
+  if (!widget::WinUtils::HasPackageIdentity()) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  ErrorResult rv;
+  RefPtr<dom::Promise> promise =
+      dom::Promise::Create(xpc::CurrentNativeGlobal(aCx), rv);
+  if (MOZ_UNLIKELY(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // A holder to pass the promise through the background task and back to
+  // the main thread when finished.
+  auto promiseHolder = MakeRefPtr<nsMainThreadPtrHolder<dom::Promise>>(
+      "IsCurrentAppPinnedToStartMenuAsync promise", promise);
+  NS_DispatchBackgroundTask(NS_NewRunnableFunction(
+      "IsCurrentAppPinnedToStartMenuAsync", [promiseHolder] {
+        IsCurrentAppPinnedToStartMenuAsyncImpl(promiseHolder);
+      }));
+  promise.forget(aPromise);
+  return NS_OK;
+}
+
+#else
+NS_IMETHODIMP
+nsWindowsShellService::EnableLaunchOnLoginMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::DisableLaunchOnLoginMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::GetLaunchOnLoginEnabledMSIXAsync(
+    const nsAString& aTaskId, JSContext* aCx,
+    /* out */ dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::PinCurrentAppToStartMenuAsync(bool aCheckOnly,
+                                                     JSContext* aCx,
+                                                     dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsWindowsShellService::IsCurrentAppPinnedToStartMenuAsync(
+    JSContext* aCx, dom::Promise** aPromise) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
+
 NS_IMETHODIMP
 nsWindowsShellService::ClassifyShortcut(const nsAString& aPath,
                                         nsAString& aResult) {
