@@ -308,7 +308,10 @@ add_task(async function test_settings_toggle_ad_and_multiple_tabs() {
  */
 add_task(async function test_shopping_settings_experiment_auto_open_disabled() {
   await SpecialPowers.pushPrefEnv({
-    set: [["browser.shopping.experience2023.autoOpen.enabled", false]],
+    set: [
+      ["browser.shopping.experience2023.autoOpen.enabled", false],
+      ["browser.shopping.experience2023.active", true],
+    ],
   });
 
   await BrowserTestUtils.withNewTab(
@@ -420,11 +423,6 @@ add_task(
               ),
               "Settings card should have a special classname with autoOpen pref enabled"
             );
-            is(
-              shoppingSettings.shoppingCardEl?.type,
-              "",
-              "shopping-card type should be default"
-            );
 
             ok(
               shoppingSettings.recommendationsToggleEl,
@@ -441,10 +439,6 @@ add_task(
               "There should be a description for the auto-open toggle"
             );
             ok(shoppingSettings.dividerEl, "There should be a divider");
-            ok(
-              shoppingSettings.sidebarEnabledStateEl,
-              "There should be a message about the sidebar active state"
-            );
 
             ok(
               shoppingSettings.optOutButtonEl,
@@ -506,11 +500,6 @@ add_task(
               ),
               "Settings card should have a special classname with autoOpen pref enabled"
             );
-            is(
-              shoppingSettings.shoppingCardEl?.type,
-              "",
-              "shopping-card type should be default"
-            );
 
             ok(
               !shoppingSettings.recommendationsToggleEl,
@@ -527,10 +516,6 @@ add_task(
               "There should be a description for the auto-open toggle"
             );
             ok(shoppingSettings.dividerEl, "There should be a divider");
-            ok(
-              shoppingSettings.sidebarEnabledStateEl,
-              "There should be a message about the sidebar active state"
-            );
 
             ok(
               shoppingSettings.optOutButtonEl,
@@ -595,10 +580,23 @@ add_task(async function test_settings_auto_open_toggle() {
       let toggleStateChangePromise = ContentTaskUtils.waitForCondition(() => {
         return !autoOpenToggle.hasAttribute("pressed");
       }, "Waiting for auto-open toggle state to be disabled");
+      let autoOpenUserEnabledPromise = ContentTaskUtils.waitForEvent(
+        content.document,
+        "autoOpenEnabledByUserChanged"
+      );
+      let activePrefChange = ContentTaskUtils.waitForCondition(
+        () =>
+          !SpecialPowers.getBoolPref("browser.shopping.experience2023.active"),
+        "Sidebar active pref should be false, but isn't"
+      );
 
       autoOpenToggle.click();
 
-      await toggleStateChangePromise;
+      Promise.all([
+        await toggleStateChangePromise,
+        await autoOpenUserEnabledPromise,
+        await activePrefChange,
+      ]);
 
       ok(
         !SpecialPowers.getBoolPref(
