@@ -261,6 +261,11 @@ var FullPageTranslationsPanel = new (class {
       const panel = wrapper.content.firstElementChild;
       wrapper.replaceWith(wrapper.content);
 
+      panel.addEventListener("command", this);
+      panel.addEventListener("click", this);
+      panel.addEventListener("popupshown", this);
+      panel.addEventListener("popuphidden", this);
+
       const settingsButton = document.getElementById(
         "translations-panel-settings"
       );
@@ -292,8 +297,9 @@ var FullPageTranslationsPanel = new (class {
         errorMessage: "full-page-translations-panel-error-message",
         errorMessageHint: "full-page-translations-panel-error-message-hint",
         errorHintAction: "full-page-translations-panel-translate-hint-action",
-        fromMenuList: "full-page-translations-panel-from",
         fromLabel: "full-page-translations-panel-from-label",
+        fromMenuList: "full-page-translations-panel-from",
+        fromMenuPopup: "full-page-translations-panel-from-menupopup",
         header: "full-page-translations-panel-header",
         intro: "full-page-translations-panel-intro",
         introLearnMoreLink:
@@ -306,6 +312,7 @@ var FullPageTranslationsPanel = new (class {
         restoreButton: "full-page-translations-panel-restore-button",
         toLabel: "full-page-translations-panel-to-label",
         toMenuList: "full-page-translations-panel-to",
+        toMenuPopup: "full-page-translations-panel-to-menupopup",
         translateButton: "full-page-translations-panel-translate",
         unsupportedHeader:
           "full-page-translations-panel-unsupported-language-header",
@@ -569,18 +576,42 @@ var FullPageTranslationsPanel = new (class {
     error.hidden = true;
     langSelection.hidden = false;
 
-    /** @type {null | LangTags} */
-    const langTags = await this.#fetchDetectedLanguages();
-    if (langTags?.isDocLangTagSupported || force) {
+    const { userLangTag, docLangTag, isDocLangTagSupported } =
+      await this.#fetchDetectedLanguages().then(langTags => langTags ?? {});
+
+    if (isDocLangTagSupported || force) {
       // Show the default view with the language selection
       const { cancelButton } = this.elements;
 
-      if (langTags?.isDocLangTagSupported) {
-        fromMenuList.value = langTags?.docLangTag ?? "";
+      if (isDocLangTagSupported) {
+        fromMenuList.value = docLangTag ?? "";
       } else {
         fromMenuList.value = "";
       }
-      toMenuList.value = langTags?.userLangTag ?? "";
+
+      if (userLangTag && userLangTag !== docLangTag) {
+        // The userLangTag is specified and does not match the doc lang tag, so we should use it.
+        toMenuList.value = userLangTag;
+      } else {
+        // No userLangTag is specified in the cache, so we will attempt to find a suitable one.
+        toMenuList.value =
+          await TranslationsParent.getTopPreferredSupportedToLang({
+            excludeLangTags: [
+              // Avoid offering to translate into the original source language.
+              docLangTag,
+              // Avoid same-language to same-language translations if possible.
+              fromMenuList.value,
+            ],
+          });
+      }
+
+      if (fromMenuList.value === toMenuList.value) {
+        // The best possible user-preferred language tag that we were able to find for the
+        // toMenuList is the same as the fromMenuList, but same-language to same-language
+        // translations are not allowed in Full Page Translations, so we will just show the
+        // "Choose a language" option in this case.
+        toMenuList.value = "";
+      }
 
       this.onChangeLanguages();
 
@@ -615,12 +646,12 @@ var FullPageTranslationsPanel = new (class {
         "full-page-translations-panel-view-unsupported-language"
       );
       let language;
-      if (langTags?.docLangTag) {
+      if (docLangTag) {
         const displayNames = new Intl.DisplayNames(undefined, {
           type: "language",
           fallback: "none",
         });
-        language = displayNames.of(langTags.docLangTag);
+        language = displayNames.of(docLangTag);
       }
       if (language) {
         document.l10n.setAttributes(
@@ -680,7 +711,8 @@ var FullPageTranslationsPanel = new (class {
     const shouldDisable =
       !docLangTag ||
       !isDocLangTagSupported ||
-      docLangTag === new Intl.Locale(Services.locale.appLocaleAsBCP47).language;
+      docLangTag ===
+        (await TranslationsParent.getTopPreferredSupportedToLang());
 
     for (const menuitem of alwaysOfferTranslationsMenuItems) {
       menuitem.setAttribute(
@@ -799,7 +831,14 @@ var FullPageTranslationsPanel = new (class {
     }
     intro.hidden = true;
     fromMenuList.value = fromLanguage;
-    toMenuList.value = toLanguage;
+    toMenuList.value = await TranslationsParent.getTopPreferredSupportedToLang({
+      excludeLangTags: [
+        // Avoid offering to translate into the original source language.
+        fromLanguage,
+        // Avoid offering to translate into current active target language.
+        toLanguage,
+      ],
+    });
     this.onChangeLanguages();
   }
 
@@ -820,7 +859,9 @@ var FullPageTranslationsPanel = new (class {
   onChangeFromLanguage(event) {
     const { target } = event;
     if (target?.value) {
-      TranslationsParent.telemetry().panel().onChangeFromLanguage(target.value);
+      TranslationsParent.telemetry()
+        .fullPagePanel()
+        .onChangeFromLanguage(target.value);
     }
     this.onChangeLanguages();
   }
@@ -833,7 +874,9 @@ var FullPageTranslationsPanel = new (class {
   onChangeToLanguage(event) {
     const { target } = event;
     if (target?.value) {
-      TranslationsParent.telemetry().panel().onChangeToLanguage(target.value);
+      TranslationsParent.telemetry()
+        .fullPagePanel()
+        .onChangeToLanguage(target.value);
     }
     this.onChangeLanguages();
   }
@@ -857,7 +900,7 @@ var FullPageTranslationsPanel = new (class {
    * within the translations panel.
    */
   onLearnMoreLink() {
-    TranslationsParent.telemetry().panel().onLearnMoreLink();
+    TranslationsParent.telemetry().fullPagePanel().onLearnMoreLink();
     FullPageTranslationsPanel.close();
   }
 
@@ -865,7 +908,7 @@ var FullPageTranslationsPanel = new (class {
    * Handler for clicking the learn more link from the gear menu.
    */
   onAboutTranslations() {
-    TranslationsParent.telemetry().panel().onAboutTranslations();
+    TranslationsParent.telemetry().fullPagePanel().onAboutTranslations();
     PanelMultiView.hidePopup(this.elements.panel);
     const window =
       gBrowser.selectedBrowser.browsingContext.top.embedderElement.ownerGlobal;
@@ -929,23 +972,25 @@ var FullPageTranslationsPanel = new (class {
     } = this.elements;
     switch (event.target.id) {
       case cancelButton.id: {
-        TranslationsParent.telemetry().panel().onCancelButton();
+        TranslationsParent.telemetry().fullPagePanel().onCancelButton();
         break;
       }
       case changeSourceLanguageButton.id: {
-        TranslationsParent.telemetry().panel().onChangeSourceLanguageButton();
+        TranslationsParent.telemetry()
+          .fullPagePanel()
+          .onChangeSourceLanguageButton();
         break;
       }
       case dismissErrorButton.id: {
-        TranslationsParent.telemetry().panel().onDismissErrorButton();
+        TranslationsParent.telemetry().fullPagePanel().onDismissErrorButton();
         break;
       }
       case restoreButton.id: {
-        TranslationsParent.telemetry().panel().onRestorePageButton();
+        TranslationsParent.telemetry().fullPagePanel().onRestorePageButton();
         break;
       }
       case translateButton.id: {
-        TranslationsParent.telemetry().panel().onTranslateButton();
+        TranslationsParent.telemetry().fullPagePanel().onTranslateButton();
         break;
       }
     }
@@ -966,11 +1011,11 @@ var FullPageTranslationsPanel = new (class {
         break;
       }
       case fromMenuList.firstChild.id: {
-        TranslationsParent.telemetry().panel().onOpenFromLanguageMenu();
+        TranslationsParent.telemetry().fullPagePanel().onOpenFromLanguageMenu();
         break;
       }
       case toMenuList.firstChild.id: {
-        TranslationsParent.telemetry().panel().onOpenToLanguageMenu();
+        TranslationsParent.telemetry().fullPagePanel().onOpenToLanguageMenu();
         break;
       }
     }
@@ -985,17 +1030,19 @@ var FullPageTranslationsPanel = new (class {
     const { panel, fromMenuList, toMenuList } = this.elements;
     switch (event.target.id) {
       case panel.id: {
-        TranslationsParent.telemetry().panel().onClose();
+        TranslationsParent.telemetry().fullPagePanel().onClose();
         this.#isPopupOpen = false;
         this.elements.error.hidden = true;
         break;
       }
       case fromMenuList.firstChild.id: {
-        TranslationsParent.telemetry().panel().onCloseFromLanguageMenu();
+        TranslationsParent.telemetry()
+          .fullPagePanel()
+          .onCloseFromLanguageMenu();
         break;
       }
       case toMenuList.firstChild.id: {
-        TranslationsParent.telemetry().panel().onCloseToLanguageMenu();
+        TranslationsParent.telemetry().fullPagePanel().onCloseToLanguageMenu();
         break;
       }
     }
@@ -1005,14 +1052,14 @@ var FullPageTranslationsPanel = new (class {
    * Handle telemetry events when the settings menu is shown.
    */
   handleSettingsPopupShownEvent() {
-    TranslationsParent.telemetry().panel().onOpenSettingsMenu();
+    TranslationsParent.telemetry().fullPagePanel().onOpenSettingsMenu();
   }
 
   /**
    * Handle telemetry events when the settings menu is hidden.
    */
   handleSettingsPopupHiddenEvent() {
-    TranslationsParent.telemetry().panel().onCloseSettingsMenu();
+    TranslationsParent.telemetry().fullPagePanel().onCloseSettingsMenu();
   }
 
   /**
@@ -1028,30 +1075,21 @@ var FullPageTranslationsPanel = new (class {
    *   True if the panel was automatically opened, otherwise false.
    * @param {boolean} telemetryData.maintainFlow
    *   Whether or not to maintain the flow of telemetry.
-   * @param {boolean} telemetryData.isFirstUserInteraction
-   *   Whether or not this is the first user interaction with the panel.
    */
   async #openPanelPopup(
     target,
-    {
-      event = null,
-      viewName = null,
-      autoShow = false,
-      maintainFlow = false,
-      isFirstUserInteraction = null,
-    }
+    { event = null, viewName = null, autoShow = false, maintainFlow = false }
   ) {
     const { panel, appMenuButton } = this.elements;
     const openedFromAppMenu = target.id === appMenuButton.id;
     const { docLangTag } = await this.#getCachedDetectedLanguages();
 
-    TranslationsParent.telemetry().panel().onOpen({
+    TranslationsParent.telemetry().fullPagePanel().onOpen({
       viewName,
       autoShow,
       docLangTag,
       maintainFlow,
       openedFromAppMenu,
-      isFirstUserInteraction,
     });
 
     this.#isPopupOpen = true;
@@ -1112,9 +1150,6 @@ var FullPageTranslationsPanel = new (class {
         gBrowser.selectedBrowser
       ).languageState;
 
-    // Store this value because it gets modified when #showDefaultView is called below.
-    const isFirstUserInteraction = !this._hasShownPanel;
-
     await this.#ensureLangListsBuilt();
 
     if (requestedTranslationPair) {
@@ -1144,7 +1179,6 @@ var FullPageTranslationsPanel = new (class {
       autoShow: reportAsAutoShow,
       viewName: requestedTranslationPair ? "revisitView" : "defaultView",
       maintainFlow: false,
-      isFirstUserInteraction,
     });
   }
 
@@ -1229,7 +1263,7 @@ var FullPageTranslationsPanel = new (class {
    * Redirect the user to about:preferences
    */
   openManageLanguages() {
-    TranslationsParent.telemetry().panel().onManageLanguages();
+    TranslationsParent.telemetry().fullPagePanel().onManageLanguages();
     const window =
       gBrowser.selectedBrowser.browsingContext.top.embedderElement.ownerGlobal;
     window.openTrustedLinkIn("about:preferences#general-translations", "tab");
@@ -1276,7 +1310,7 @@ var FullPageTranslationsPanel = new (class {
     const toggledOn =
       TranslationsParent.toggleAlwaysTranslateLanguagePref(langTags);
     TranslationsParent.telemetry()
-      .panel()
+      .fullPagePanel()
       .onAlwaysTranslateLanguage(docLangTag, toggledOn);
     this.#updateSettingsMenuLanguageCheckboxStates();
     await this.#doPageAction(pageAction);
@@ -1287,7 +1321,9 @@ var FullPageTranslationsPanel = new (class {
    */
   async onAlwaysOfferTranslations() {
     const toggledOn = TranslationsParent.toggleAutomaticallyPopupPref();
-    TranslationsParent.telemetry().panel().onAlwaysOfferTranslations(toggledOn);
+    TranslationsParent.telemetry()
+      .fullPagePanel()
+      .onAlwaysOfferTranslations(toggledOn);
   }
 
   /**
@@ -1304,7 +1340,7 @@ var FullPageTranslationsPanel = new (class {
     const toggledOn =
       TranslationsParent.toggleNeverTranslateLanguagePref(docLangTag);
     TranslationsParent.telemetry()
-      .panel()
+      .fullPagePanel()
       .onNeverTranslateLanguage(docLangTag, toggledOn);
     this.#updateSettingsMenuLanguageCheckboxStates();
     await this.#doPageAction(pageAction);
@@ -1320,7 +1356,9 @@ var FullPageTranslationsPanel = new (class {
     const toggledOn = await TranslationsParent.getTranslationsActor(
       gBrowser.selectedBrowser
     ).toggleNeverTranslateSitePermissions();
-    TranslationsParent.telemetry().panel().onNeverTranslateSite(toggledOn);
+    TranslationsParent.telemetry()
+      .fullPagePanel()
+      .onNeverTranslateSite(toggledOn);
     this.#updateSettingsMenuSiteCheckboxStates();
     await this.#doPageAction(pageAction);
   }
@@ -1387,7 +1425,60 @@ var FullPageTranslationsPanel = new (class {
    * @param {CustomEvent} event
    */
   handleEvent = event => {
+    const target = event.target;
+    let { id } = target;
+
+    // If a menuitem within a menulist is the target, it will not have an id,
+    // so we want to grab the closest relevant id.
+    if (!id) {
+      id = target.closest("[id]")?.id;
+    }
+
     switch (event.type) {
+      case "command": {
+        switch (id) {
+          case "translations-panel-settings":
+            this.openSettingsPopup(target);
+            break;
+          case "full-page-translations-panel-from-menupopup":
+            this.onChangeFromLanguage(event);
+            break;
+          case "full-page-translations-panel-to-menupopup":
+            this.onChangeToLanguage(event);
+            break;
+          case "full-page-translations-panel-restore-button":
+            this.onRestore(event);
+            break;
+          case "full-page-translations-panel-cancel":
+          case "full-page-translations-panel-dismiss-error":
+            this.onCancel(event);
+            break;
+          case "full-page-translations-panel-translate":
+            this.onTranslate(event);
+            break;
+          case "full-page-translations-panel-change-source-language":
+            this.onChangeSourceLanguage(event);
+            break;
+        }
+        break;
+      }
+      case "click": {
+        switch (id) {
+          case "full-page-translations-panel-intro-learn-more-link":
+          case "full-page-translations-panel-unsupported-learn-more-link":
+            this.onLearnMoreLink();
+            break;
+          default:
+            this.handlePanelButtonEvent(event);
+        }
+        break;
+      }
+      case "popupshown":
+        this.handlePanelPopupShownEvent(event);
+        break;
+      case "popuphidden":
+        this.handlePanelPopupHiddenEvent(event);
+        break;
       case "TranslationsParent:OfferTranslation": {
         if (Services.wm.getMostRecentBrowserWindow()?.gBrowser === gBrowser) {
           this.open(event, /* reportAsAutoShow */ true);
@@ -1395,7 +1486,30 @@ var FullPageTranslationsPanel = new (class {
         break;
       }
       case "TranslationsParent:LanguageState": {
-        const { actor } = event.detail;
+        const { actor, reason } = event.detail;
+
+        const innerWindowId =
+          gBrowser.selectedBrowser.browsingContext.top.embedderElement
+            .innerWindowID;
+
+        this.console?.debug("TranslationsParent:LanguageState", {
+          reason,
+          currentId: innerWindowId,
+          originatorId: actor.innerWindowId,
+        });
+
+        if (innerWindowId !== actor.innerWindowId) {
+          // The id of the currently active tab does not match the id of the tab that was active when the event was dispatched.
+          // This likely means that the tab was changed after the event was dispatched, but before it was received by this class.
+          //
+          // Keep in mind that there is only one instance of this class (FullPageTranslationsPanel) for each open Firefox window,
+          // but there is one instance of the TranslationsParent actor for each open tab within a Firefox window. As such, it is
+          // possible for a tab-specific actor to fire an event that is received by the window-global panel after switching tabs.
+          //
+          // Since the dispatched event did not originate in the currently active tab, we should not react to it any further.
+          return;
+        }
+
         const {
           detectedLanguages,
           requestedTranslationPair,
@@ -1511,7 +1625,7 @@ var FullPageTranslationsPanel = new (class {
             break;
           case "engine-load-failure":
             this.#showEngineError(actor).catch(viewError =>
-              this.console.error(viewError)
+              this.console?.error(viewError)
             );
             break;
           default:
