@@ -24,7 +24,7 @@ if (AppConstants.platform == "macosx") {
 add_setup(async function () {
   // Add a default engine with suggestions, to avoid hitting the network when
   // fetching them.
-  let defaultEngine = await SearchTestUtils.promiseNewSearchEngine({
+  let defaultEngine = await SearchTestUtils.installOpenSearchEngine({
     url: getRootDirectory(gTestPath) + TEST_ENGINE_BASENAME,
     setAsDefault: true,
   });
@@ -39,6 +39,15 @@ add_setup(async function () {
   await SpecialPowers.pushPrefEnv({
     set: [["browser.urlbar.suggest.searches", true]],
   });
+
+  // When typing `@`, we are getting token alias engine results and restrict
+  // keyword results. This increases the results being returned.
+  // Currently, the result limit is 10, and at this limit, the alias engine is
+  // cut off. We need to extend the limit to 12 to be able to test all the
+  // results.
+  await SpecialPowers.pushPrefEnv({
+    set: [["browser.urlbar.maxRichResults", 12]],
+  });
 });
 
 // Simple test that tries different variations of an alias, without reverting
@@ -410,11 +419,21 @@ add_task(async function clickAndFillAlias() {
   let testEngineItem;
   for (let i = 0; !testEngineItem; i++) {
     let details = await UrlbarTestUtils.getDetailsOfResultAt(window, i);
-    Assert.equal(
-      details.displayed.title,
-      `Search with ${details.searchParams.engine}`,
-      "The result's title is set correctly."
-    );
+
+    if (details.result.type == UrlbarUtils.RESULT_TYPE.RESTRICT) {
+      Assert.equal(
+        details.displayed.title,
+        `Search with ${details.result.payload.l10nRestrictKeyword}`,
+        "The result's title is set correctly."
+      );
+    } else {
+      Assert.equal(
+        details.displayed.title,
+        `Search with ${details.searchParams.engine}`,
+        "The result's title is set correctly."
+      );
+    }
+
     Assert.ok(!details.action, "The result should have no action text.");
     if (details.searchParams && details.searchParams.keyword == ALIAS) {
       testEngineItem = await UrlbarTestUtils.waitForAutocompleteResultAt(
@@ -597,7 +616,10 @@ add_task(async function hiddenEngine() {
   // Checks that the default engine appears in the urlbar's popup.
   for (let i = 0; i < UrlbarTestUtils.getResultCount(window); i++) {
     let details = await UrlbarTestUtils.getDetailsOfResultAt(window, i);
-    if (defaultEngine.name == details.searchParams.engine) {
+    if (
+      details.result.type != UrlbarUtils.RESULT_TYPE.RESTRICT &&
+      defaultEngine.name == details.searchParams.engine
+    ) {
       foundDefaultEngineInPopup = true;
       break;
     }
@@ -619,7 +641,10 @@ add_task(async function hiddenEngine() {
   foundDefaultEngineInPopup = false;
   for (let i = 0; i < UrlbarTestUtils.getResultCount(window); i++) {
     let details = await UrlbarTestUtils.getDetailsOfResultAt(window, i);
-    if (defaultEngine.name == details.searchParams.engine) {
+    if (
+      details.result.type != UrlbarUtils.RESULT_TYPE.RESTRICT &&
+      defaultEngine.name == details.searchParams.engine
+    ) {
       foundDefaultEngineInPopup = true;
       break;
     }
@@ -662,7 +687,10 @@ add_task(async function nonPrefixedKeyword() {
   // Checks that the default engine appears in the urlbar's popup.
   for (let i = 0; i < UrlbarTestUtils.getResultCount(window); i++) {
     let details = await UrlbarTestUtils.getDetailsOfResultAt(window, i);
-    if (details.searchParams.engine === name) {
+    if (
+      details.result.type != UrlbarUtils.RESULT_TYPE.RESTRICT &&
+      details.searchParams.engine === name
+    ) {
       foundEngineInPopup = true;
       break;
     }
