@@ -547,9 +547,9 @@ async function focusAndWaitForFieldsIdentified(browserOrContext, selector) {
  * Run the task and wait until the autocomplete popup is opened.
  *
  * @param {object} browser A xul:browser.
- * @param {Function} taskFn Task that will trigger the autocomplete popup
+ * @param {Function} taskFunction Task that will trigger the autocomplete popup
  */
-async function runAndWaitForAutocompletePopupOpen(browser, taskFn) {
+async function runAndWaitForAutocompletePopupOpen(browser, taskFunction) {
   info("runAndWaitForAutocompletePopupOpen");
   let popupShown = BrowserTestUtils.waitForPopupEvent(
     browser.autoCompletePopup,
@@ -557,7 +557,7 @@ async function runAndWaitForAutocompletePopupOpen(browser, taskFn) {
   );
 
   // Run the task will open the autocomplete popup
-  await taskFn();
+  await taskFunction();
 
   await popupShown;
 }
@@ -574,35 +574,22 @@ async function waitForPopupEnabled(browser) {
   );
 }
 
-// Wait for the popup state change notification to happen in a child process.
-function waitPopupStateInChild(bc, messageName) {
-  return SpecialPowers.spawn(bc, [messageName], expectedMessage => {
-    return new Promise(resolve => {
-      const { AutoCompleteChild } = ChromeUtils.importESModule(
-        "resource://gre/actors/AutoCompleteChild.sys.mjs"
-      );
+// Wait for the popup state change notification to happen.
+async function waitForAutoCompletePopupOpen(browser, taskFunction) {
+  const popupShown = BrowserTestUtils.waitForPopupEvent(
+    browser.autoCompletePopup,
+    "shown"
+  );
 
-      let listener = {
-        popupStateChanged: name => {
-          if (name != expectedMessage) {
-            info("Expected " + expectedMessage + " but received " + name);
-            return;
-          }
+  if (taskFunction) {
+    await taskFunction();
+  }
 
-          AutoCompleteChild.removePopupStateListener(listener);
-          resolve();
-        },
-      };
-      AutoCompleteChild.addPopupStateListener(listener);
-    });
-  });
+  return popupShown;
 }
 
 async function openPopupOn(browser, selector) {
-  let childNotifiedPromise = waitPopupStateInChild(
-    browser,
-    "AutoComplete:PopupOpened"
-  );
+  const popupOpenPromise = waitForAutoCompletePopupOpen(browser);
   await SimpleTest.promiseFocus(browser);
 
   await runAndWaitForAutocompletePopupOpen(browser, async () => {
@@ -613,14 +600,11 @@ async function openPopupOn(browser, selector) {
     }
   });
 
-  await childNotifiedPromise;
+  await popupOpenPromise;
 }
 
 async function openPopupOnSubframe(browser, frameBrowsingContext, selector) {
-  let childNotifiedPromise = waitPopupStateInChild(
-    frameBrowsingContext,
-    "AutoComplete:PopupOpened"
-  );
+  const popupOpenPromise = waitForAutoCompletePopupOpen(browser);
 
   await SimpleTest.promiseFocus(browser);
 
@@ -632,7 +616,7 @@ async function openPopupOnSubframe(browser, frameBrowsingContext, selector) {
     }
   });
 
-  await childNotifiedPromise;
+  await popupOpenPromise;
 }
 
 async function closePopup(browser) {
@@ -641,10 +625,6 @@ async function closePopup(browser) {
     return;
   }
 
-  let childNotifiedPromise = waitPopupStateInChild(
-    browser,
-    "AutoComplete:PopupClosed"
-  );
   let popupClosePromise = BrowserTestUtils.waitForPopupEvent(
     browser.autoCompletePopup,
     "hidden"
@@ -655,16 +635,10 @@ async function closePopup(browser) {
   });
 
   await popupClosePromise;
-  await childNotifiedPromise;
 }
 
 async function closePopupForSubframe(browser, frameBrowsingContext) {
-  let childNotifiedPromise = waitPopupStateInChild(
-    browser,
-    "AutoComplete:PopupClosed"
-  );
-
-  let popupClosePromise = BrowserTestUtils.waitForPopupEvent(
+  const popupClosePromise = BrowserTestUtils.waitForPopupEvent(
     browser.autoCompletePopup,
     "hidden"
   );
@@ -674,7 +648,6 @@ async function closePopupForSubframe(browser, frameBrowsingContext) {
   });
 
   await popupClosePromise;
-  await childNotifiedPromise;
 }
 
 function emulateMessageToBrowser(name, data) {
@@ -682,6 +655,7 @@ function emulateMessageToBrowser(name, data) {
     gBrowser.selectedBrowser.browsingContext.currentWindowGlobal.getActor(
       "FormAutofill"
     );
+
   return actor.receiveMessage({ name, data });
 }
 
@@ -905,24 +879,17 @@ async function setStorage(...items) {
   }
 }
 
-function verifySectionAutofillResult(sections, expectedSectionsInfo) {
-  sections.forEach((section, index) => {
-    const expectedSection = expectedSectionsInfo[index];
+function verifySectionAutofillResult(section, result, expectedSection) {
+  const fieldDetails = section.fieldDetails;
+  const expectedFieldDetails = expectedSection.fields;
 
-    const fieldDetails = section.fieldDetails;
-    const expectedFieldDetails = expectedSection.fields;
-
-    info(`verify autofill section[${index}]`);
-
-    fieldDetails.forEach((field, fieldIndex) => {
-      const expeceted = expectedFieldDetails[fieldIndex];
-
-      Assert.equal(
-        field.element.value,
-        expeceted.autofill ?? "",
-        `Autofilled value for element(id=${field.element.id}, field name=${field.fieldName}) should be equal`
-      );
-    });
+  fieldDetails.forEach((field, fieldIndex) => {
+    const expected = expectedFieldDetails[fieldIndex];
+    Assert.equal(
+      result.get(field.elementId).value,
+      expected.autofill ?? "",
+      `Autofilled value for element(identifier:${field.identifier}, field name:${field.fieldName}) should be equal`
+    );
   });
 }
 
@@ -946,7 +913,7 @@ function verifySectionFieldDetails(sections, expectedSectionsInfo) {
       `Expected field count.`
     );
 
-    fieldDetails.forEach((field, fieldIndex) => {
+    fieldDetails.forEach((fieldDetail, fieldIndex) => {
       const expectedFieldDetail = expectedFieldDetails[fieldIndex];
 
       const expected = {
@@ -955,28 +922,28 @@ function verifySectionFieldDetails(sections, expectedSectionsInfo) {
           section: "",
           contactType: "",
           addressType: "",
-          credentialType: "",
+          part: undefined,
         },
         ...expectedSection.default,
         ...expectedFieldDetail,
       };
 
-      const keys = new Set([...Object.keys(field), ...Object.keys(expected)]);
-      [
-        "identifier",
-        "autofill",
-        "elementWeakRef",
-        "confidence",
+      const keys = [
+        "reason",
+        "section",
+        "contactType",
+        "addressType",
+        "fieldName",
         "part",
-      ].forEach(k => keys.delete(k));
+      ];
 
       for (const key of keys) {
         const expectedValue = expected[key];
-        const actualValue = field[key];
+        const actualValue = fieldDetail[key];
         Assert.equal(
           actualValue,
           expectedValue,
-          `${key} should be equal, expect ${expectedValue}, got ${actualValue}`
+          `[${fieldDetail.fieldName}]: ${key} should be equal, expect ${expectedValue}, got ${actualValue}`
         );
       }
     });
@@ -1101,73 +1068,48 @@ async function add_heuristic_tests(
     }
 
     await BrowserTestUtils.withNewTab(TEST_URL, async browser => {
-      await SpecialPowers.spawn(
-        browser,
-        [
-          {
-            testPattern,
-            verifySection: verifySectionFieldDetails.toString(),
-            verifyAutofill: options.testAutofill
-              ? verifySectionAutofillResult.toString()
-              : null,
-          },
-        ],
-        async obj => {
-          const { FormLikeFactory } = ChromeUtils.importESModule(
-            "resource://gre/modules/FormLikeFactory.sys.mjs"
-          );
-          const { FormAutofillHandler } = ChromeUtils.importESModule(
-            "resource://gre/modules/shared/FormAutofillHandler.sys.mjs"
-          );
+      await SpecialPowers.spawn(browser, [], async function () {
+        const elements = Array.from(
+          content.document.querySelectorAll("input, select")
+        );
+        // Focus on each field in the test document to trigger autofill field detection
+        // on all the fields.
+        elements.forEach(element => element.focus());
+      });
 
-          const elements = Array.from(
-            content.document.querySelectorAll("input, select")
-          );
+      const actor =
+        browser.browsingContext.currentWindowGlobal.getActor("FormAutofill");
+      await BrowserTestUtils.waitForCondition(() => {
+        return actor.getSections().length == testPattern.expectedResult.length;
+      }, "Expected section count.");
 
-          // Bug 1834768. We should simulate user behavior instead of
-          // using internal APIs.
-          const forms = elements.reduce((acc, element) => {
-            const formLike = FormLikeFactory.createFromField(element);
-            if (!acc.some(form => form.rootElement === formLike.rootElement)) {
-              acc.push(formLike);
-            }
-            return acc;
-          }, []);
+      // Verify the identified fields in each section.
+      const sections = actor.getSections();
+      verifySectionFieldDetails(sections, testPattern.expectedResult);
 
-          const sections = forms.flatMap(form => {
-            const handler = new FormAutofillHandler(form);
-            handler.collectFormFields(false /* ignoreInvalid */);
-            return handler.sections;
-          });
-
-          Assert.equal(
-            sections.length,
-            obj.testPattern.expectedResult.length,
-            "Expected section count."
-          );
-
-          // eslint-disable-next-line no-eval
-          let verify = eval(`(() => {return (${obj.verifySection});})();`);
-          verify(sections, obj.testPattern.expectedResult);
-
-          if (obj.verifyAutofill) {
-            for (const section of sections) {
-              if (!section.isValidSection()) {
-                continue;
-              }
-
-              section.focusedInput = section.fieldDetails[0].element;
-              await section.autofillFields(
-                section.getAdaptedProfiles([obj.testPattern.profile])[0]
-              );
-            }
-
-            // eslint-disable-next-line no-eval
-            verify = eval(`(() => {return (${obj.verifyAutofill});})();`);
-            verify(sections, obj.testPattern.expectedResult);
+      // Verify the autofilled value.
+      if (options.testAutofill) {
+        for (let index = 0; index < sections.length; index++) {
+          const section = sections[index];
+          // We do not autofill for sections that are invalid.
+          if (!section.isValidSection()) {
+            continue;
           }
+
+          // Trigger autofill from the first element of this section
+          const elementId = section.fieldDetails[0].elementId;
+          const result = await actor.autofillFields(
+            elementId,
+            testPattern.profile
+          );
+
+          verifySectionAutofillResult(
+            section,
+            result,
+            testPattern.expectedResult[index]
+          );
         }
-      );
+      }
     });
 
     if (testPattern.prefs) {
