@@ -983,7 +983,7 @@ async function assertShowPreviewTooltip(view, target) {
  *        The DOM Element on which a tooltip should appear
  */
 async function assertTooltipHiddenOnMouseOut(tooltip, target) {
-  // The tooltip actually relies on mousemove events to check if it sould be hidden.
+  // The tooltip actually relies on mousemove events to check if it should be hidden.
   const mouseEvent = new target.ownerDocument.defaultView.MouseEvent(
     "mousemove",
     {
@@ -998,6 +998,167 @@ async function assertTooltipHiddenOnMouseOut(tooltip, target) {
   ok(!tooltip.isVisible(), "The tooltip is hidden on mouseout");
 }
 
+/**
+ * Check the content of a `var()` tooltip on a given rule and property name.
+ *
+ * @param {CssRuleView} view
+ * @param {String} ruleSelector
+ * @param {String} propertyName
+ * @param {Object} tooltipExpected
+ * @param {String} tooltipExpected.header: The HTML for the top section
+ *        (might be the only section when the variable is not a registered property and
+ *        there is no starting-style, nor computed value).
+ * @param {Array<String>} tooltipExpected.headerClasses: Classes applied on the header element
+ *        (no need to include `variable-value` which is always added).
+ * @param {String} tooltipExpected.computed: The HTML for the computed value section.
+ * @param {Array<String>} tooltipExpected.computedClasses: Classes applied on the computed value element.
+ * @param {Integer} tooltipExpected.index: The index in the property value for the variable
+ *        element we want to check. Defaults to 0 so we can quickly check values when only
+ *        one variable is used.
+ * @param {Boolean} tooltipExpected.isMatched: Is the element matched or unmatched, defaults
+ *        to true.
+ * @param {String} tooltipExpected.startingStyle: The HTML for the starting-style section.
+ *        Pass undefined if the tooltip isn't supposed to have a `@starting-style` section.
+ * @param {Array<String>} tooltipExpected.startingStyleClasses: Classes applied on the
+ *        starting-style value element.
+ * @param {Object} tooltipExpected.registeredProperty: Object whose properties should match
+ *        the displayed registered property fields, e.g:
+ *        {syntax:`"&lt;color&gt;"`, inherits:"true", "initial-value": "10px"}
+ *        The properties values are the HTML of the dd elements.
+ *        Pass undefined if the tooltip isn't supposed to have a @property section.
+ */
+async function assertVariableTooltipForProperty(
+  view,
+  ruleSelector,
+  propertyName,
+  {
+    computed,
+    computedClasses = ["theme-fg-color1"],
+    header,
+    headerClasses = ["theme-fg-color1"],
+    index = 0,
+    isMatched = true,
+    registeredProperty,
+    startingStyle,
+    startingStyleClasses = ["theme-fg-color1"],
+  }
+) {
+  // retrieve tooltip target
+  const variableEl = await waitFor(
+    () =>
+      getRuleViewProperty(
+        view,
+        ruleSelector,
+        propertyName
+      ).valueSpan.querySelectorAll(".ruleview-variable,.ruleview-unmatched")[
+        index
+      ]
+  );
+
+  if (isMatched) {
+    ok(
+      !variableEl.classList.contains("ruleview-unmatched"),
+      `CSS variable #${index} for ${propertyName} in ${ruleSelector} is matched`
+    );
+  } else {
+    ok(
+      variableEl.classList.contains("ruleview-unmatched"),
+      `CSS variable #${index} for ${propertyName} in ${ruleSelector} is unmatched`
+    );
+  }
+
+  const previewTooltip = await assertShowPreviewTooltip(view, variableEl);
+  const valueEl = previewTooltip.panel.querySelector(".variable-value");
+  const computedValueEl = previewTooltip.panel.querySelector(".computed div");
+  const startingStyleEl = previewTooltip.panel.querySelector(
+    ".starting-style div"
+  );
+  const registeredPropertyEl = previewTooltip.panel.querySelector(
+    ".registered-property dl"
+  );
+  is(
+    valueEl?.innerHTML,
+    header,
+    `CSS variable #${index} preview tooltip has expected header text for ${propertyName} in ${ruleSelector}`
+  );
+  Assert.deepEqual(
+    [...valueEl.classList],
+    ["variable-value", ...headerClasses],
+    `CSS variable #${index} preview tooltip has expected classes for ${propertyName} in ${ruleSelector}`
+  );
+
+  if (typeof computed !== "string") {
+    is(
+      computedValueEl,
+      null,
+      `CSS variable #${index} preview tooltip doesn't have computed value section for ${propertyName} in ${ruleSelector}`
+    );
+  } else {
+    is(
+      computedValueEl?.innerHTML,
+      computed,
+      `CSS variable #${index} preview tooltip has expected computed value section for ${propertyName} in ${ruleSelector}`
+    );
+    Assert.deepEqual(
+      [...computedValueEl.classList],
+      computedClasses,
+      `CSS variable #${index} preview tooltip has expected classes on computed value for ${propertyName} in ${ruleSelector}`
+    );
+  }
+
+  if (!registeredProperty) {
+    is(
+      registeredPropertyEl,
+      null,
+      `CSS variable #${index} preview tooltip doesn't have registered property section for ${propertyName} in ${ruleSelector}`
+    );
+  } else {
+    const dts = registeredPropertyEl.querySelectorAll("dt");
+    const registeredPropertyEntries = Object.entries(registeredProperty);
+    is(
+      dts.length,
+      registeredPropertyEntries.length,
+      `CSS variable #${index} preview tooltip has the expected number of element in the registered property section for ${propertyName} in ${ruleSelector}`
+    );
+    for (let i = 0; i < registeredPropertyEntries.length; i++) {
+      const [label, value] = registeredPropertyEntries[i];
+      const dt = dts[i];
+      const dd = dt.nextElementSibling;
+      is(
+        dt.innerText,
+        `${label}:`,
+        `CSS variable #${index} preview tooltip has expected ${label} registered property element for ${propertyName} in ${ruleSelector}`
+      );
+      is(
+        dd.innerHTML,
+        value,
+        `CSS variable #${index} preview tooltip has expected HTML for ${label} registered property element for ${propertyName} in ${ruleSelector}`
+      );
+    }
+  }
+
+  if (!startingStyle) {
+    is(
+      startingStyleEl,
+      null,
+      `CSS variable #${index} preview tooltip doesn't have a starting-style section for ${propertyName} in ${ruleSelector}`
+    );
+  } else {
+    is(
+      startingStyleEl?.innerHTML,
+      startingStyle,
+      `CSS variable #${index} preview tooltip has expected starting-style section for ${propertyName} in ${ruleSelector}`
+    );
+    Assert.deepEqual(
+      [...startingStyleEl.classList],
+      startingStyleClasses,
+      `CSS variable #${index} preview tooltip has expected classes on starting-style value for ${propertyName} in ${ruleSelector}`
+    );
+  }
+
+  await assertTooltipHiddenOnMouseOut(previewTooltip, variableEl);
+}
+
 /**
  * Get the text displayed for a given DOM Element's textContent within the
  * markup view.
