@@ -347,14 +347,16 @@ class WalkerActor extends Actor {
     return "[WalkerActor " + this.actorID + "]";
   }
 
-  getDocumentWalker(node, skipTo) {
-    // Allow native anon content (like <video> controls) if preffed on
-    const filter = this.showAllAnonymousContent
+  getDocumentWalkerFilter() {
+    // Allow native anonymous content (like <video> controls) if preffed on
+    return this.showAllAnonymousContent
       ? allAnonymousContentTreeWalkerFilter
       : standardTreeWalkerFilter;
+  }
 
+  getDocumentWalker(node, skipTo) {
     return new DocumentWalker(node, this.rootWin, {
-      filter,
+      filter: this.getDocumentWalkerFilter(),
       skipTo,
       showAnonymousContent: true,
     });
@@ -899,7 +901,7 @@ class WalkerActor extends Actor {
   }
 
   /**
-   * Returns the raw children of the DOM node, with anon content filtered as needed
+   * Returns the raw children of the DOM node, with anonymous content filtered as needed
    * @param Node rawNode.
    * @param boolean includeAssigned
    *   Whether <slot> assigned children should be returned. See
@@ -2199,6 +2201,19 @@ class WalkerActor extends Actor {
    *    See https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver#MutationRecord
    */
   onMutations(mutations) {
+    // Don't send a mutation event if the mutation target would be ignored by the walker
+    // filter function.
+    const documentWalkerFilter = this.getDocumentWalkerFilter();
+    if (
+      mutations.every(
+        mutation =>
+          documentWalkerFilter(mutation.target) ===
+          nodeFilterConstants.FILTER_SKIP
+      )
+    ) {
+      return;
+    }
+
     // Notify any observers that want *all* mutations (even on nodes that aren't
     // referenced).  This is not sent over the protocol so can only be used by
     // scripts running in the server process.
@@ -2325,6 +2340,13 @@ class WalkerActor extends Actor {
    */
   onAnonymousrootcreated(event) {
     const root = event.target;
+
+    // Don't trigger a mutation if the document walker would filter out the element.
+    const documentWalkerFilter = this.getDocumentWalkerFilter();
+    if (documentWalkerFilter(root) === nodeFilterConstants.FILTER_SKIP) {
+      return;
+    }
+
     const parent = this.rawParentNode(root);
     if (!parent) {
       // These events are async. The node might have been removed already, in
