@@ -223,7 +223,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -256,7 +255,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl("docs/Web/JavaScript/Reference/Errors/Bad_radix"),
@@ -287,7 +285,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl("docs/Web/JavaScript/Reference/Errors/Read-only"),
@@ -318,7 +315,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -351,7 +347,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -391,7 +386,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -431,7 +425,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -465,7 +458,6 @@ const expectedPageErrors = new Map([
       warning: true,
       info: false,
       sourceId: null,
-      lineText: "function a() { return; 1 + 1; }",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -492,7 +484,6 @@ const expectedPageErrors = new Map([
       warning: false,
       info: false,
       sourceId: null,
-      lineText: "{let a, a;}",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: mdnUrl(
@@ -528,7 +519,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: undefined,
@@ -559,7 +549,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: undefined,
@@ -597,7 +586,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       exceptionDocURL: undefined,
@@ -640,7 +628,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
@@ -685,7 +672,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
@@ -738,7 +724,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
@@ -776,7 +761,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
@@ -814,7 +798,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
@@ -850,7 +833,6 @@ const expectedPageErrors = new Map([
       error: true,
       warning: false,
       info: false,
-      lineText: "",
       lineNumber: NUMBER_REGEX,
       columnNumber: NUMBER_REGEX,
       innerWindowID: NUMBER_REGEX,
